import pytest
from pkglts.config_management import Config
from pkglts.option.plugin_project.option import OptionPluginProject


@pytest.fixture()
def opt():
    return OptionPluginProject('plugin_project')


def test_update_parameters(opt):
    cfg = {}
    opt.update_parameters(cfg)
    assert len(cfg['plugin_project']) == 1


def test_config_check_plugin_names(opt):
    cfg = Config(dict(plugin_project={'plugin_name': "walou"}))
    assert 'plugin_project.plugin_name' not in opt.check(cfg)

    for pkg in ('1mypkg', ' mypkg', '1', '1.mypkg',
                ' .mypkg', '.mypkg', 'None.mypkg', 'oa.o.mypkg'):
        cfg = Config(dict(plugin_project={'plugin_name': pkg}))
        assert 'plugin_project.plugin_name' in opt.check(cfg)


def test_require_option(opt):
    assert len(tuple(opt.require_option())) == 3


def test_require(opt):
    cfg = Config()
    opt.update_parameters(cfg)

    assert len(tuple(opt.require(cfg))) == 1



"""
Example plot
============

This is just an example.
"""
from math import pi

import matplotlib.pyplot as plt
import numpy as np

x = np.linspace(0,100,200)
y = np.sin(x / 50 * pi)

fig, ax = plt.subplots(1, 1, figsize=(10, 6))

ax.plot(x, y)

fig.tight_layout()
plt.show()



from pathlib import Path

from pkglts.local import src_dir
from pkglts.option_object import Option
from pkglts.version import __version__


class OptionSrc(Option):
    def version(self):
        return __version__

    def root_dir(self):
        return Path(__file__).parent

    def update_parameters(self, cfg):
        sec = dict(
            namespace_method="pkg_util",
        )
        cfg[self._name] = sec

    def check(self, cfg):
        invalids = []

        if cfg[self._name]['namespace_method'] not in ("pkg_util", "setuptools", "P3.3>"):
            invalids.append("src.namespace_method")

        return invalids

    def require_option(self):
        return ['base']

    def environment_extensions(self, cfg):
        return {
            "src_pth": src_dir(cfg).as_posix()
        }



import json
from pathlib import Path

import pytest
from pkglts.config import pkg_cfg_file, pkglts_dir
from pkglts.config_management import Config, get_pkg_config, write_pkg_config
from pkglts.manage import init_pkg, regenerate_option, regenerate_package
from pkglts.small_tools import ensure_created, rmdir


def addendum(init_file):
    """ modify init_file in first pkglts div
    """
    with open(init_file, 'r') as fhr:
        lines = fhr.read().splitlines()

    lines.insert(1, "addendum")

    with open(init_file, 'w') as fhw:
        fhw.write("\n".join(lines))
        fhw.write("\n")


@pytest.fixture()
def tmp_pths():
    pth = Path('toto_mg_rg')
    ensure_created(pth)
    init_pkg(pth)
    with open((pth / pkglts_dir / pkg_cfg_file), 'r') as fhr:
        pkg_cfg = json.load(fhr)

    pkg_cfg['base'] = dict(pkgname='toto',
                           namespace=None,
                           authors=[('moi', 'moi@email.com')],
                           url=None)
    pkg_cfg['src'] = dict(namespace_method="pkg_util")

    cfg = Config(pkg_cfg)
    write_pkg_config(cfg, pth)
    regenerate_package(cfg, pth)

    init_file = pth / "src/toto/__init__.py"

    yield pth, init_file

    rmdir(pth)


def test_regenerate_pass(tmp_pths):
    tmp_dir, init_file = tmp_pths
    with open(tmp_dir / pkglts_dir / pkg_cfg_file, 'r') as fhr:
        pkg_cfg = json.load(fhr)

    cfg = Config(pkg_cfg)
    regenerate_package(cfg, tmp_dir)
    assert init_file.exists()


def test_regenerate_check_pkg_cfg_validity(tmp_pths):
    tmp_dir, init_file = tmp_pths
    with open(tmp_dir / pkglts_dir / pkg_cfg_file, 'r') as fhr:
        pkg_cfg = json.load(fhr)

    pkg_cfg['base']['pkgname'] = '1toto'

    cfg = Config(pkg_cfg)
    assert not regenerate_package(cfg, tmp_dir)


def test_regenerate_handle_conflicts_keep(tmp_pths, mocker):
    tmp_dir, init_file = tmp_pths
    cfg = get_pkg_config(tmp_dir)

    init_file.write_text("modified")

    mocker.patch('pkglts.manage.get_user_permission', return_value=False)
    regenerate_package(cfg, tmp_dir)

    assert init_file.read_text() == "modified"


def test_regenerate_handle_conflicts_overwrite(tmp_pths, mocker):
    tmp_dir, init_file = tmp_pths
    cfg = get_pkg_config(tmp_dir)

    addendum(init_file)

    mocker.patch('pkglts.manage.get_user_permission', return_value=True)
    regenerate_package(cfg, tmp_dir)

    assert "modified" not in init_file.read_text()


def test_regenerate_handle_global_overwrite(tmp_pths):
    tmp_dir, init_file = tmp_pths
    cfg = get_pkg_config(tmp_dir)

    addendum(init_file)

    regenerate_package(cfg, tmp_dir, overwrite=True)

    assert "modified" not in init_file.read_text()


def test_regenerate_new_files_do_not_generate_conflicts(tmp_pths):
    tmp_dir, init_file = tmp_pths
    cfg = get_pkg_config(tmp_dir)

    new_pth = tmp_dir / "src/toto/new_file.py"
    new_pth.write_text("txt = 'addendum'")

    regenerate_package(cfg, tmp_dir)

    assert new_pth.exists()
    assert new_pth.read_text() == "txt = 'addendum'"


def test_regenerate_remove_user_files_do_not_generate_conflicts(tmp_pths):
    tmp_dir, init_file = tmp_pths
    cfg = get_pkg_config(tmp_dir)

    new_pth = tmp_dir / "src/toto/new_file.py"
    new_pth.write_text("txt = 'addendum'")

    regenerate_package(cfg, tmp_dir)

    new_pth.unlink()

    regenerate_package(cfg, tmp_dir)
    assert not new_pth.exists()


def test_regenerate_remove_tpl_files_do_not_generate_conflicts(tmp_pths):
    tmp_dir, init_file = tmp_pths
    cfg = get_pkg_config(tmp_dir)

    regenerate_package(cfg, tmp_dir)

    init_file.unlink()

    regenerate_package(cfg, tmp_dir)
    assert init_file.exists()


def test_regenerate_fail_if_permanent_section_ids_have_been_modified(tmp_pths):
    tmp_dir, init_file = tmp_pths
    cfg = get_pkg_config(tmp_dir)

    with open(init_file, 'a') as fhw:
        fhw.write("\n# {# pkglts, test\na = 1\n# #}\n")

    with pytest.raises(UserWarning):
        regenerate_package(cfg, tmp_dir, overwrite=True)


def test_regenerate_option_fails_if_option_not_available(tmp_pths):
    tmp_dir, init_file = tmp_pths
    cfg = get_pkg_config(tmp_dir)

    regenerate_option(cfg, 'base', tmp_dir)

    with pytest.raises(KeyError):
        regenerate_option(cfg, "toto", tmp_dir)



import pytest

from pkglts.option.doc import fmt_badge


def test_fmt_badge_works_correctly():
    badge = "mybadge"
    url = "myurl"
    txt = "mytxt"

    sec = fmt_badge(badge, url, txt, 'md')
    assert badge in sec
    assert url in sec
    assert txt in sec

    sec = fmt_badge(badge, url, txt, 'rst')
    assert badge in sec
    assert url in sec
    assert txt in sec

    with pytest.raises(UserWarning):
        fmt_badge(badge, url, txt, 'docx')



import pytest
from pkglts.config_management import Config
from pkglts.option.github.option import OptionGithub


@pytest.fixture()
def opt():
    return OptionGithub('github')


def test_update_parameters(opt):
    cfg = {}
    opt.update_parameters(cfg)
    assert len(cfg['github']) == 3


def test_config_check_project_exists(opt):
    cfg = Config(dict(github={'owner': "", 'project': "", "url": ""}))
    assert 'github.project' in opt.check(cfg)


def test_require_option(opt):
    assert len(tuple(opt.require_option())) == 1


def test_require(opt):
    cfg = Config()
    opt.update_parameters(cfg)

    assert len(tuple(opt.require(cfg))) == 0
